; ===========================================================================
;  crt0.s
;  ------
; Initialization file for GCC compiler for Motorola MPC5xx PowerPC
; Microcontroller.  Linking this will allow software to boot from flash.
;
; ===========================================================================

; #include "ppc-asm.h"

	.file	"crt0.S"
	.text

	.globl	_start
	.type	_start,@function

	.globl _hwreset
	.globl gInterruptVectorTable
	.globl gInterruptVectorTableEnd


not finished.  see init.s



_start:
	bl	.Laddr			/* get current address */
.Laddr:
	mflr	r4			/* real address of .Laddr */
	lwz	r0,(.Lstk-.Laddr)(r4)	/* stack address location */
	mr	sp,r0			/* use user defined stack */

	addi	sp,sp,-4		/* make sure we don't overwrite debug mem */
	lis	r0,0
	stw	r0,0(sp)		/* clear back chain */
	stwu	sp,-64(sp)		/* push another stack frame */
#if 0
/* coppied from Motorola code */
   /*------------------------------------------------------------------------- */
   /* Explicitly program the Machine State Register (MSR). Clear the MSR(IP)  */
   /* bit to ensure the vector interrupt table starts at 0x00000000 for all */
   /* exceptions that occur after the vector table is relocated to SDRAM */
   /* further down in this code. The Machine Check Exception is enabled.  */
   /*------------------------------------------------------------------------- */

   mfmsr    r3
   ori      r3,r3,0x1000    /* Set ME flags */
   mtmsr    r3

   /*############################################# */
   /* Call 82xx_init to finish chip initialization. */
   /*############################################# */


/*   bl mpc82xx_init */       /* User specific init code  */

   /*############################################################ */
   /* Initialize the stack for the main program's code execution */
   /*############################################################ */

/*   bl init_stack */    /* Initialize the stack */

#endif

	/* Let her rip */
	bl	FUNC_NAME(main)

	/* return value from main is argument to exit */
	bl	FUNC_NAME(exit)
	trap


/*######################################################################## */
/* Function: mpc82xx_init */
/* */
/* Description: This function initializes the rest of the mpc82xx with the  */
/*              exception of the CPM. */
/* */
/*  History: */
/* */
/*  1/8/99    jay       */
/* */
/*########################################################################         */

mpc82xx_init:

   mfspr    r31,LR        /* Save the Link Register value. The link register's */
                          /* value will be restored so that this function  */
                          /* can return to the calling address. */
#if 0
   addis    r4,0,0x0471   /* IMMR base addr = 0x04700000+10000. We add  */
                          /* 0x10000 because using relative addressing */
                          /* in load and store instructions only allow a */
                          /* offset from the base of +/-32767. */
   addis    r5,0,0x0470

   addis    r3,0,0x0F01
   stw      r5,IMMR(r3)   /* The default IMMR base address was 0x0F000000 as  */
                          /* originally programmed into the Hard Reset  */
                          /* Configuration Word. */
#endif
   /******************************************** */
   /* Main System Clock Configuration Registers  */
   /******************************************** */

   /*------------------------------------------------------------------------- */
   /* We only need to program the System Clock Control Register (SCCR). The  */
   /* System Clock Mode Register (SCMR) doesn't need to be programmed here */
   /* because the MODCLK_HI bits in the Hard Reset Configuration Word and the */
   /* MODCK pins dictate the values in the SCCR during power-on reset. */
   /*------------------------------------------------------------------------- */

   /*------------------------------------------------------------------------- */
   /* Program the System Clock Control Register (SCCR). */
   /* */
   /* - Bits 0 - 28 Reserved. Clear to 0. */
   /* */
   /* - CLPD (CPM Low Power Disable) = 0 =  */
   /* */
   /*     CPM does not enter low power mode when the core enters low power  */
   /*     mode. */
   /* */
   /* - DFBRG (Division Factor of BRGCLK) = 01 - Divide by 8. */
   /* */
   /*------------------------------------------------------------------------- */

   addis    r3,0,0x0000
   ori      r3,r3,0x0001   /* SCCR = 0x00000001 */
   stw      r3,SCCR(r4)

   /*~~~~~~~~~~~~~~~~~~~~ */
   /* Initialize the SIU */
   /*~~~~~~~~~~~~~~~~~~~~ */

   bl       init_siu

   /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
   /* Initialize the memory controller and SDRAM */
   /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

   bl       init_memc

   /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
   /* Vector Interrupt Table Relocation to address 0x00000000 */
   /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

/*   bl       vector_reloc */

   /*~~~~~~~~~~~~~~~~ */
   /* Initialize MMU */
   /*~~~~~~~~~~~~~~~~ */

/*   bl       init_bat_mmu */

   /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
   /* Initialize Instruction and Caches */
   /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

   bl       init_caches

   /*~~~~~~~~~~~~~~~~~~~~ */
   /* end of mpc82xx_init */
   /*~~~~~~~~~~~~~~~~~~~~ */

   /*~~~~~~~~~~~~~~~~~~~~ */
   /* end of mpc82xx_init */
   /*~~~~~~~~~~~~~~~~~~~~ */

   mtspr    LR,r31      /* restore original Link Register value  */

   bclr     20,0        /* jump unconditionally to effective address in Link */
                        /* register */



/*######################################################################## */
/* Function: init_stack */
/* */
/* Description:  */
/* */
/*   This function initializes the C code in uart.c's stack. */
/*   NOTE:  This code is compiler-specific. Diab uses R1 as the stack pointer. */
/*   R1 is loaded with the "top of stack" address. This code should only be  */
/*   called during startup and during the exception recovery routines. */
/* */
/* */
/*  History: */
/* */
/*  1/9/99    jay       */
/* */
/*########################################################################         */

#if 0
init_stack:

   /*------------------------------------------------------------------------ */
   /* SP_INIT this value is defined in init8260.lnx which is the linker  */
   /* file. */
   /*------------------------------------------------------------------------ */

   addis    r1,r0,__SP_INIT@h
   ori      r1,r1,__SP_INIT@l

   /*---------------------------------------------------------------------- */
   /* load the stack pointer 72 bytes down from top of stack to ensure one */
   /* stack frame of safety margin.  */
   /*---------------------------------------------------------------------- */

   stwu  r0,-72(r1)

   bclr  20,0           /* jump unconditionally to effective address in Link */
                        /* register */
#endif

/*######################################################################## */
/* Function: init_siu */
/* */
/* Description: Initializes the System Interface Unit */
/* */
/*  History: */
/* */
/*  1/9/99    jay       */
/* */
/*########################################################################         */

init_siu:

   /*----------------------------------------------------------------------- */
   /* Program the System Protection Control Register with the following  */
   /* values: */
   /* */
   /* - SWTC (Software Watchdog Timer Count) = 0xFFFF = programmed to max */
   /*   count value. */
   /* */
   /* - BMT (Bus Monitor Timing) = 0xFF = Set to max timeout period. */
   /* */
   /* - PBME (60x Bus Monitor Enable) = 1 = enabled. */
   /* */
   /* - LBME (Local Bus Monitor Enable) = 1 = enabled. */
   /*   */
   /* - Bits 26-28 Reserved. Set to 0. */
   /* */
   /* - SWE (Software Watchdog Enabled) = 0 = disabled for now. User will  */
   /*   have to enable this in a non-debug application. */
   /* */
   /* - SWRI (Software Watchdog Reset/Interrupt Select) = 1 = Software  */
   /*   watchdog timeout or bus monitor time-out causes a soft reset. */
   /* */
   /* - SWP (Software Watchdog Prescale) = 1 = clock is prescaled. */
   /*----------------------------------------------------------------------- */

   addis    r3,0,0xFFFF
   ori      r3,r3,0xFFC3   /* SYPCR = 0xFFFFFFC3 */
   stw      r3,SYPCR(r4)

   /*------------------------------------------------------------------------- */
   /* Program the Bus Configuration Register. The details are as follows: */
   /* */
   /* - EBM (External Bus Mode) = 0 = Single MPC8260 bus mode is assumed. */
   /* */
   /* - APD (Address Phase delay) = 001 =  */
   /* */
   /*     One address tenure wait states for address operations initiated by  */
   /*     a 60x bus master. */
   /*                               */
   /* - L2C (Secondary Cache Controller) = 0 = No secondary cache controller */
   /*                                          is assumed. */
   /* */
   /* - L2D (L2 cache hit Delay) = 000 = This is a don't care because we're  */
   /*                                    not using the L2 cache. */
   /* */
   /* - PLDP (PipeLine maximum Depth) = 0 = The pipeline max depth is one. */
   /* */
   /* - Bits 9-11 = Reserved. Set to 0. */
   /* */
   /* - ETM (Compatibility Mode enable) = 1 = Extended transfer mode is */
   /*                                         enabled. */
   /* */
   /* - LETM (Local Bus Compatibility Mode Enable) = 1 =  */
   /* */
   /*     Extended transfer mode is enabled on the local bus. */
   /* */
   /* - EPAR (Even Parity) = 0 = This is a don't care. */
   /* */
   /* - LEPAR (Local Bus Even Parity) = 0 = This is a don't care. */
   /* */
   /* - Bits 16-20 = Reserved. Set to 0. */
   /* */
   /* - EXDD (External Master Delay Disable) = 0 =  */
   /* */
   /*     The memroy controller inserts one wait state between the assertion  */
   /*     of TS and the assertion of CS when an external master accesses an  */
   /*     address space controlled by the memory controller. */
   /* */
   /* - Bits 22-26 = Reserved. Set to 0. */
   /* */
   /* - ISPS (Internal Space Port Size) = 0 =  */
   /* */
   /*     MPC8260 acts as a 64-bit slave to external master accesses to it's  */
   /*     internal space. */
   /* */
   /* - Bits 28-31 = Reserved. Set to 0. */
   /*                                               */
   /*------------------------------------------------------------------------- */

   addis    r3,0,0x100c
   stw      r3,BCR(r4)

   /*------------------------------------------------------------------------- */
   /* Program the 60x Bus Arbiter Configuration Register. The details are as */
   /* follows: */
   /* */
   /* - Bits 0-1 = Reserved. Set to 0.  */
   /* */
   /* - DBGD (Data Bus Grant Delay) = Minimum of zero wait states for PowerPC */
   /*                                 master-initiated data operations. This  */
   /*                                 is the minimum delay between TS/ and  */
   /*                                 DBG/. */
   /* */
   /* - Bits 3 = Reserved. Set to 0. */
   /* */
   /* - PRKM (Parking Master) = 0010 = CPM is at a low request level for the  */
   /*                                  parked master. */
   /* */
   /*------------------------------------------------------------------------- */

   addi     r3,0,0x0002
   stb      r3,PPC_ACR(r4)


   /*------------------------------------------------------------------------- */
   /* Program the 60x Bus Arbitration-Level[High] Register. The priority for  */
   /* potential bus masters are defined by locating the value associated with */
   /* a particular master from the PRKM field in the PPC_ACR register and  */
   /* inserting it's value in the priority field. Priority field 0 is the */
   /* highest priority and the lowest is Priority field 15 in the PPC_ALRH */
   /* register. */
   /* */
   /* - Priority Field 0 = 0000 = CPM high request level */
   /* */
   /* - Priority Field 1 = 0001 = CPM middle request level */
   /* */
   /* - Priority Field 2 = 0010 = CPM low request level */
   /* */
   /* - Priority Field 3 = 0110 = Internal Core */
   /* */
   /* - Priority Field 4 = 0111 = External Master 1 */
   /* */
   /* - Priority Field 5 = 1000 = External Master 2 */
   /* */
   /* - Priority Field 6 = 1001 = External Master 3 */
   /* */
   /* - Priority Field 7 = 0011 = Reserved */
   /* */
   /*------------------------------------------------------------------------- */

   addis    r3,0,0x0126
   ori      r3,r3,0x7893
   stw      r3,PPC_ALRH(r4)


   /*------------------------------------------------------------------------ */
   /* */
   /* First program the SIUMCR. The details are as follows: */
   /* */
   /* - BBD (Bus Busy Disable) = 0 = [ABB/]/[IRQ2] pin is ABB/ and [DBB/]/IRQ2  */
   /*                                pin is DBB/. */
   /* */
   /* - ESE (External Snoop Enable = 0 = [GPL/]/[IRQ1/] pin is IRQ1/ */
   /* */
   /* - PBSE (Parity Byte Select Enable) = 0 = Parity byte select is disabled. */
   /* */
   /* - CDIS (Core DISable) = 0 = The PowerQUICCII CPU core is enabled. */
   /* */
   /* - DPPC (Data Parity Pins Configuration) = 10 =  */
   /* */
   /*    Gives the following pin meanings:   DP(0)|RSRV/|EXT_BR2/  = RSRV/ */
   /*                                        DP(1)|IRQ1/|EXT_BG2/  = IRQ1/ */
   /*                                        DP(2)|TLBISYNC/|IRQ2/ = TLBISYNC/ */
   /*                                        DP(3)|IRQ3/           = IRQ3/     */
   /*                                        DP(4)|IRQ4/           = IRQ4/     */
   /*                                        DP(5)|TBEN/|IRQ5      = TBEN/ */
   /*                                        DP(6)|CSE(0)|IRQ6/    = CSE(0) */
   /*                                        DP(7)|CSE(1)|IRQ7/    = CSE(1) */
   /* */
   /* - L2CPC (L2 Cache Pins Configuration) = 00 = */
   /* */
   /*    Gives the following pin meanings:   CI/|BADDR(29)|IRQ2/ = CI/  */
   /*                                        WT/|BADDR(30)|IRQ3/ = WT/  */
   /*                                        L2_HIT/|IRQ4        = L2_HIT/  */
   /*                                                                      */
   /*                                        CPU_BG/|BADDR(31)     */
   /*                                        |IRQ5/              = CPU_BG/ */
   /*    JMS here, 13/3/1999, modifying the L2CPC to = 10 = because the PILOT */
   /*        revision of board uses the BADDR's to address the FLASH. */
   /* */
   /* */
   /* - LBPC (Local Bus Pins Configuration) = 00 =  */
   /* */
   /*    Local Bus pins function as local bus. */
   /* */
   /* - APPC (Address Parity Pins Configuration) = 10 =  */
   /* */
   /*    Gives the following pin meanings: MODCK1|AP(1)|TC(0)  = BNKSEL(0) */
   /*                                      MODCK2|AP(2)|TC(1)  = BNKSEL(1) */
   /*                                      MODCK3|AP(3)|TC(2)  = BNKSEL(2) */
   /*                                      IRQ7/|APE/|INT_OUT/ = IRQ7/+INT_OUT/ */
   /*                                      CS11/|AP(0)         = CS11/ */
   /* */
   /* - CS10PC (Chip Select 10-pin Configuration) = 00 =  */
   /* */
   /*    CS10/|BCTL1/|DBG_DIS/ = CS10/ */
   /* */
   /* - BCTLC (Buffer Control Configuration) = 00 =  */
   /* */
   /*    BCTL0 pin is used as W|R/ control. BCTL1 is used as OE/ control if  */
   /*    BCTL1 was connected to it's alternate pin. */
   /* */
   /* - MMR (Mask Masters Requests) = 00 = No masking on bus request lines. */
   /* */
   /* - LPBSE (Local Bus Parity Byte Select Enable) = 0 */
   /*  */
   /*    Parity byte select is disabled. */
   /* */
   /* Bits 19-31 are reserved and set to 0. */
   /* */
   /*------------------------------------------------------------------------ */

   addis    r3,0,0x0A20
   ori      r3,r3,0x0000
   stw      r3,SIUMCR(r4)


   /*-------------------------------------------------------------------------- */
   /* Program the 60x Bus Transfer Error Status and Control Register 1. The */
   /* details are as follows: */
   /* */
   /* - DMD (Data Errors Disable) = 1 =  */
   /* */
   /*     Disable all data errors on the 60x bus. Also parity single and double */
   /*     ECC error. */
   /* */
   /* - All other bits are either status or reserved bits. All reserved bits  */
   /*   should be set to 0. */
   /*-------------------------------------------------------------------------- */

   addis    r3,0,0x0000
   ori      r3,r3,0x4000
   stw      r3,TESCR1(r4)

   /*-------------------------------------------------------------------------- */
   /* Program the Local Bus Transfer Error Status and Control Register 1. The */
   /* details are as follows: */
   /* */
   /* - DMD (Data Errors Disable) = 1 =  */
   /* */
   /*     Disable parity errors on the Local bus.  */
   /* */
   /* - All other bits are either status or reserved bits. All reserved bits  */
   /*   should be set to 0. */
   /*-------------------------------------------------------------------------- */

   addis    r3,0,0x0000
   ori      r3,r3,0x4000
   stw      r3,LTESCR1(r4)


   /*------------------------------------------------------------------------- */
   /* First, program the Memory Periodic Timer Prescaler Register (MPTPR). */
   /* Finding the value to this ties in with the desired SDRAM Refresh */
   /* Timer (PSRT) value and the required Refresh Command interval for  */
   /* refreshing each row. The Refresh Command interval is found by  */
   /* determining the number of rows on the SDRAM device. In this case it's  */
   /* 2048. The "total time between refresh" value in the AC characteristic  */
   /* section of the data sheet is 32.8 msec. This value is divided by the  */
   /* number of rows to give the number of Refresh commands that needs to be */
   /* sent in a 32.8 msec interval. This value is 32.8 Msec/2048 = 16.02 usec. */
   /* Due to probable contention from time to time with other memory  */
   /* controller bus requests, lets make the refresh command interval to be */
   /* around 15.5 usec. */
   /* */
   /* Now let's divide the input VADs system clock by 64 (somewhat arbitrary). */
   /* If this is our prescaler value then the input frequency to the Refresh */
   /* timer is 1.031 Mhz. Using the formula: */
   /* */
   /*        Timer Period = PSRT */
   /*                       ---- */
   /*                       F(MPTC) */
   /* */
   /*        - Where F(MPTC) is the output from the MPT Prescaler block or */
   /*          66 Mhz/64 = 1.031 Mhz. */
   /* */
   /*        - Timer Period = 15.5 usec */
   /* */
   /* The PSRT value would be approximately 16. */
   /* */
   /* The clock distribution block diagram looks like this: */
   /* */
   /*   ----------      -------------         -----------    ------------- */
   /*   | Clock   |     |   MPT     |         | SDRAM   |    | Refresh   | */
   /*   | Gen.    |-----| Prescaler |---------| Refresh |----| Command   | */
   /*   | 66 Mhz  |     -------------         | Timer   |    | Logic in  | */
   /*   | on Vads |                           -----------    | Mem.Cont. | */
   /*   | Board   |                                          ------------- */
   /*   ----------- */
   /*       */
   /*------------------------------------------------------------------------- */

   addi     r5,0,0x4000      /* load 0x40 or 64 into the PTP field of MPTPR */
   sth      r5,MPTPR(r4)     /* store half word - bits[16-31] */



   /*----------------------- */
   /* return from init_siu */
   /*----------------------- */

   bclr  20,0           /* jump unconditionally to effective address in Link */
                        /* register */




/*######################################################################## */
/* Function: init_memc */
/* */
/* Description:  */
/* */
/*  The following registers directly control the memory controller's  */
/*  operation: */
/* */
/*  BR0-BR11 - Base Register Banks 0-11 */
/*  OR0-OR11 - Option Register Banks 0-11 */
/*  PSDMR    - 60x bus SDRAM machine mode register */
/*  LSDMR    - Local bus SDRAM machine mode register */
/*  MAMR     - UPMA mode register */
/*  MBMR     - UPMB mode register */
/*  MCMR     - UPMC mode register */
/*  MDR      - Memory data register */
/*  MAR      - Memory address register */
/*  MPTPR    - Memory periodic timer pre-scaler register */
/*  PURT     - 60x bus assigned UPM refresh timer */
/*  PSRT     - 60x bus assigned SDRAM refresh timer */
/*  LURT     - Local Bus assigned UPM refresh timer */
/*  LSRT     - Local Bus assigned SDRAM refresh timer */
/* */
/*  This example will program the following registers. The rest will remain at */
/*  their default values. */
/* */
/*  BR0      - Base Register for Flash Memory */
/*  OR0      - Option Register for Flash Memory */
/*  BR1      - Base Register for BCSR (Board Control and Status Registers) */
/*  OR1      - Option Register for BCSR */
/*  BR2      - Base Register for 60x SDRAM */
/*  OR2      - Option Register for 60x SDRAM */
/*  BR3      - Base Register for 60x Local Bus SDRAM */
/*  OR3      - Option Register for 60x Local Bus SDRAM */
/*  PSDMR    - 60x bus SDRAM machine mode register */
/*  LSDMR    - Local bus SDRAM machine mode register */
/*  MPTPR    - Memory periodic timer pre-scaler register */
/*  PSRT     - 60x bus assigned SDRAM refresh timer */
/*  LSRT     - Local Bus assigned SDRAM refresh timer */
/*               */
/* */
/*  History: */
/* */
/*  1/9/99    jay       */
/* */
/*########################################################################         */

init_memc:

   mfspr    r30,LR        /* Save the Link Register value. The link register's */
                          /* value will be restored so that this function  */
                          /* can return to the calling address. */

/*   bl    init_flash  */      /* 8 Mbyte of flash memory */

   bl    init_bcsr         /* Board Control and Status Registers */

/*   bl    init_local_sdram */ /* Local Bus SDRAM */

   bl    init_60x_sdram    /* Main 60x Bus SDRAM */

   /*----------------------- */
   /* return from init_memc */
   /*----------------------- */

   mtspr    LR,r30      /* restore original Link Register value  */

   bclr  20,0           /* jump unconditionally to effective address in Link */
                        /* register */


#if 0
/*######################################################################## */
/* Function: vector_reloc */
/* */
/* Description: This function relocates the vector table to physical addr */
/*              0x00000000 */
/* */
/*  History: */
/* */
/*  1/9/99    jay       */
/* */
/*########################################################################         */

vector_reloc:

   /*----------------------------------------------------------------------- */
   /* This section of code calculates the actual number of entries from the  */
   /* vector interrupt table. */
   /*----------------------------------------------------------------------- */

   addis       r5,0,VectorTable@h         /* point R5 to table beginning */
   ori         r5,r5,VectorTable@l

   addis       r6,0,VectorTableEnd@h      /* point R6 to end paramenter table */
   ori         r6,r6,VectorTableEnd@l

   sub         r3,r6,r5                 /* Get total # of bytes */
   srawi       r3,r3,2                  /* divide by 4 for actual # of entries */

   /*------------------------------------------------------------------------ */
   /* load start address for low memory vector location. Note that the          */
   /* transfer starts with a 256 byte offset. This is because the first */
   /* 256 bytes is the hard reset configuration area and it's definition */
   /* is in config.s. VectorTable: label actually starts with the 0x0100 */
   /* exception. There is no need to relocate the hard reset configuration */
   /* because there is not a 0x0000 exception. */
   /*------------------------------------------------------------------------ */

   addi    r2,0,0x0100

   /*-------------------------- */
   /* Start transferring table */
   /*-------------------------- */

vloop:

   addic.  r3,r3,-1     /* decrement count value */
   lwz     r6,0(r5)     /* grab the vector table value */
   addi    r5,r5,4      /* move source index to next word */
   stw     r6,0(r2)     /* write value to low memory */
   addi    r2,r2,4      /* move destination index to next word */
   bc      4,2,vloop    /* if not zero, loop */


   /*-------------------------- */
   /* return from vector_reloc */
   /*-------------------------- */

   bclr  20,0           /* jump unconditionally to effective address in Link */
                        /* register */
#endif

/*########################################################################### */
/* Function: init_caches */
/* */
/* Description: Initializes the Data/Instruction Caches.  */
/* */
/*  History: */
/* */
/*  1/9/99    jay       */
/* */
/*###########################################################################         */


init_caches:

   mfspr    r30,LR        /* Save the Link Register value */

   /*------------------------------ */
   /* Enable the Instruction Cache  */
   /*------------------------------ */

   bl    EnableAndInvalidateInstCache

   /*----------------------- */
   /* Enable the Data Cache  */
   /*----------------------- */

   bl    EnableAndInvalidateDataCache

   /*---------------------------- */
   /* return from init_cache */
   /*---------------------------- */

   mtspr    LR,r30      /* restore original Link Register value  */

   bclr  20,0           /* jump unconditionally to effective address in Link */
                        /* register */



/*########################################################################### */
/* Function: EnableAndInvalidateDataCache */
/* */
/* Description: This function is used to initialize the Data cache to a start */
/*              state. This is done by ensuring that data cache is enabled by */
/*              explicitly enabling it and invalidating all cache lines or  */
/*              blocks. Note that by executing this function, all modified */
/*              blocks will be destoyed. */
/* History: */
/* */
/* 1/9/99    Initial    jay   */
/* */
/*########################################################################         */

EnableAndInvalidateDataCache:

   mfspr r5,HID0
   ori   r5,r5,0x4400   /* Set DCE and DCFI bit */
   andi. r6,r5,0xFBFF   /* Clear the DCFI bit for the final store */
   mtspr HID0,r5
   isync
   sync

   mtspr HID0,r6        /* Store the final value */
   isync
   sync

   bclr  20,0           /* jump unconditionally to effective address in Link */
                        /* register */


/*########################################################################### */
/* Function: EnableAndInvalidateInstCache */
/* */
/* Description: This function first enables the instruction cache and then */
/*              sets the instruction cache flash invalidate (ICFI) bit in  */
/*              the HID0 register. This establishes a start state for the  */
/*              cache. */
/* */
/* History: */
/* */
/* 1/9/99    Initial    jay   */
/* */
/*########################################################################         */

EnableAndInvalidateInstCache:

   mfspr r5,HID0
   ori   r5,r5,0x8800   /* Set the ICE and ICFI bit */
   andi. r6,r5,0xF7FF   /* clear the ICFI bit for the final store */
   mtspr HID0,r5
   isync
   sync

   mtspr HID0,r6        /* Do the final store */
   isync
   sync
   bclr  20,0           /* jump unconditionally to effective address in Link */
                        /* register */

/*######################################################################## */
/* Function: init_bcsr */
/* */
/* Description: This function programs Base Register 1 and Option Register 1 */
/*              designating bank 1 for BCSR0, BCSR1, and BCSR2 on the VADS */
/*              board. BCSR stands for Board Control and Status Register. */
/*              This space is treated as general I/O. Programming the  */
/*              following 2 registers describes how the MPC8260 will inter- */
/*              operate with this memory space. */
/* */
/*  History: */
/* */
/*  1/9/99    jay       */
/* */
/*########################################################################         */

init_bcsr:


   /*------------------------------------------------------------------------- */
   /* Base Register 1 (BR1): Bank 1 is assigned to the Board Control and */
   /*                        Status Registers (BCSRs). There are 3  */
   /*                        that resides on the MPC8260 VADS board.  */
   /*                        The particulars are defined here.  */
   /* */
   /* BA (Base Address) = 0x0450+0b for a total of 17 address bits. This value */
   /*                     represents the upper 17 bits of the base address. */
   /* */
   /* Bits 17-18 reserved. = 00 */
   /* */
   /* PS (Port Size) = 11b = 32 bit port size */
   /* */
   /* DECC (Data Error Correction and Checking) = 00 = Data errors checking */
   /*                                                  Disabled. */
   /* */
   /* WP (Write Protect) = 0 = both read and write accesses are allowed */
   /* */
   /* MS (Machine Select) = 000 = General Purpose Chip Select Machine (GPCM) */
   /*                             for 60x bus Selected */
   /* */
   /* EMEMC (External Memory Controller Enable) = 0 = Accesses are handled by */
   /*                                                 the memory controller */
   /*                                                 according to MSEL. */
   /* */
   /* ATOM (Atomic Operation) = 00 = The address space controlled by the  */
   /*                                memory controller bank is not used for */
   /*                                atomic operations. */
   /* */
   /* DR (Delayed Read) = 0 = Normal operation. */
   /* */
   /* V (Valid Bit) =  1 = Valid bit set */
   /*------------------------------------------------------------------------- */

   addis    r3,0,0x0450     /* R3 holds the value temporarily */
   ori      r3,r3,0x1801

   /*------------------------------------------------------------------------- */
   /* Option Register 1 (OR1) for GPCM use: further BCSR definitions */
   /* */
   /* AM (Address Mask) = 0xFFFF +1b = We've masked the upper 17 bits which  */
   /*                                  which defines a 32 Kbyte memory block. */
   /* */
   /* Bits 17-19 Reserved - set to 000. */
   /* */
   /* CSNT (Chip Select Negation Time) = 0 = CS/|WE/ are negated normally. */
   /* */
   /* ACS (Address To Chip-Select Setup) = 00 = CS/ is output at the same  */
   /*                                           time as the addr lines. */
   /* */
   /* Bit 23 Reserved - set to 0. */
   /* */
   /* SCY (Cycle Length In Clocks) = 0001 = Add a 1 clock cycle wait state */
   /* */
   /* SETA (External Transfer Acknowledge) = 0 = PSDVAL/ is generated  */
   /*                                            internally by the memory  */
   /*                                            controller unless GTA/ is  */
   /*                                            asserted earlier externally. */
   /* */
   /* TRLX (Timing Relaxed) = 0 = Normal timing is generated by the GPCM. */
   /* */
   /* EHTR (Extended Hold Time On Read Accesses) = 0 = Normal timing is  */
   /*                                                  generated by the memory */
   /*                                                  controller */
   /* */
   /* Bit 31 Reserved - set to 0. */
   /*------------------------------------------------------------------------- */

   addis    r5,0,0xFFFF     /* R5 holds the value temporarily */
   ori      r5,r5,0x8010

   /*------------------ */
   /* Write the values */
   /*------------------ */

   stw        r5,OR1(r4)
   stw        r3,BR1(r4)


   bclr  20,0    /* jump unconditionally to effective address in Link */
                 /* register */


/*######################################################################## */
/* Function: init_local_sdram */
/* */
/* Description: This function programs the 4 Mbytes of SDRAM on the Local */
/*              Bus on the VADS board. This memory functions as storage for */
/*              connection tables and data buffers for the CPM peripherals. */
/*              Base and Option Register 4 are used. SDRAM Machine #2 */
/*              and Chip Select #4 is used in the memory controller. */
/* */
/*  History: */
/* */
/*  1/9/99    jay       */
/* */
/*########################################################################         */

init_local_sdram:

   /*################# */
   /* Program Refresh */
   /*################# */

   /*------------------------------------------------------------------------- */
   /* Program the Local Bus Assigned SDRAM Refresh Timer (LSRT). Note that the */
   /* MPTPR register was programmed in the init_siu function which also */
   /* dictates the frequency to the LSRT block. Because the frequency rating */
   /* for the local bus SDRAM is same as the 60x SDRAM DIMM, only the LSRT */
   /* needs to be programmed and it is going to be the same as the PSRT */
   /* register. */
   /*------------------------------------------------------------------------- */

   addi     r5,0,0x0010      /* load 0x10 or 16  */
   stb      r5,LSRT(r4)      /* store byte - bits[24-31] */


   /*######################## */
   /* Program Bank Registers */
   /*######################## */


   /*------------------------------------------------------------------------- */
   /* Base Register 4 (BR4): Bank 4 is assigned to the 4 Mbyte Local SDRAM  */
   /*                        bank that resides on the MPC8260 VADS board. The  */
   /*                        particulars are defined here.  */
   /* */
   /* - BA (Base Address) = 0x0400+0b for a total of 17 address bits. This  */
   /*                       value represents the upper 17 bits of the base  */
   /*                       address. */
   /* */
   /* - Bits 17-18 reserved. = 00 = cleared to 0. */
   /* */
   /* - PS (Port Size) = 11b = 32 bit port size */
   /* */
   /* - DECC (Data Error Correction and Checking) = 00 = Data errors checking */
   /*                                                    Disabled. */
   /* */
   /* - WP (Write Protect) = 0 = both read and write accesses are allowed */
   /* */
   /* - MS (Machine Select) = 011 = SDRAM Machine for Local bus Selected */
   /* */
   /* - EMEMC (External Memory Controller Enable) = 0 =  */
   /* */
   /*    Accesses are handled by the memory controller according to MSEL. */
   /* */
   /* - ATOM (Atomic Operation) = 00 =  */
   /* */
   /*    The address space controlled by the memory controller bank is not  */
   /*    used for atomic operations. */
   /* */
   /* - DR (Delayed Read) = 0 = Normal operation. */
   /* */
   /* - V (Valid Bit) =  1 = Valid bit set */
   /*------------------------------------------------------------------------- */

   addis    r3,0,0x0400     /* R3 holds the value temporarily */
   ori      r3,r3,0x1861

   /*------------------------------------------------------------------------- */
   /* Option Register 4 (OR4) for SDRAM Machine use: further SDRAM definitions */
   /* */
   /* - USDAM (Upper SDRAM Address Mask) = 11111b =  */
   /* */
   /*    We've masked all 5 bits which defines a maximum 128 Mbyte block Note */
   /*    that since the maximum SDRAM block size is 128 Mbyte, this field */
   /*    should always be 11111b. */
   /* */
   /* - SDAM (SDRAM Address Mask) = 1111100b = 4 Mbyte block. */
   /* */
   /* - LSDAM (Lower SDRAM Address Mask) = 00000b = Minimum of 1 Mbyte size. */
   /* */
   /* - BPD (Banks Per Device) = 00 = 2 internal banks per device. */
   /* */
   /* - ROWST (Row Start Address Bit) = 101 = A11 being the row start address */
   /*                                         bit. */
   /* */
   /* - Bit 22 Reserved - set to 0. */
   /* */
   /* - NUMR (Number of Row Address Lines) = 010 = 11 row address lines. */
   /* */
   /* - PMSEL (Page Mode Select) = 0 = back-to-back page mode (normal  */
   /*                                  operation). */
   /* */
   /* - IBID (Internal bank interleaving within same device disable) = 0 = */
   /*  */
   /*    Bank interleaving allowed. */
   /* */
   /* - AACKR (AACK/ release) = 0 = Normal operation. */
   /* */
   /* - Bits 29-31 Reserved - set to 0. */
   /*------------------------------------------------------------------------- */

   addis    r5,0,0xFFC0     /* R5 holds the value temporarily */
   ori      r5,r5,0x1480

   /*------------------ */
   /* Write the values */
   /*------------------ */

   stw        r5,OR4(r4)
   stw        r3,BR4(r4)


   /*########################################## */
   /* Perform Initialization sequence to SDRAM */
   /*########################################## */

   /*------------------------------------------------------------------------- */
   /* Program the Local SDRAM Mode Registr (LSDMR). This register is used */
   /* to configure operations pertaining to SDRAM on the Local bus. Turn off  */
   /* refresh services and change the SDRAM operation to "Precharge all banks". */
   /* Then do a single write to an arbitrary location. Writing 0 to address */
   /* 0 will do the trick. */
   /* */
   /* - Bit 0 is reserved. Set to 0. */
   /* */
   /* - RFEN (Refresh Enable) = 0 = Refresh services not required. This bit  */
   /*                               will be set later in this function as a  */
   /*                               last step. */
   /*  */
   /* - OP (SDRAM Operation) = 101 = Precharge all banks. */
   /* */
   /* - SDAM (Address Multiplex Size = 000 =  */
   /* */
   /*   Coming up the value for this field is one of the most confusing */
   /*   and non-intuitive steps in the SDRAM programming process. This is how */
   /*   it works... The SDRAM device is a 4 Mbit chip set that has a data   */
   /*   width of 32 bits or 4 bytes so the SDRAM will ignore the least  */
   /*   significant 2 bits. Given this information and knowing that the  */
   /*   number of row address lines is 11 and the column addresses is 8 and  */
   /*   also knowing that the row addresses must be multiplexed, write out the  */
   /*   following: */
   /* */
   /*   addresses ignored by SDRAM : A30 A31 */
   /* */
   /*   column addresses: A22 A23 A24 A25 A26 A27 A28 A29 */
   /* */
   /*   row addresses:   A11 A12 A13 A14 A15 A16 A17 A18 A19 A20 A21 */
   /* */
   /*   When the memory controller multiplexes the addresses it puts the  */
   /*   column addresses out on the actual physical pins that matches it's */
   /*   corresponding effective address bits. */
   /* */
   /*   According to the SDRAM Address Multiplexing table in the memory  */
   /*   controller section of the manual, SDAM selection of 000 fits the  */
   /*   address layout given above. Because the row is what is multiplexed  */
   /*   on designated column address pins, the row addresses are put out on the */
   /*   following column pins: */
   /* */
   /*                                   8 column addresses */
   /*                                   | */
   /*   column ------------->A19 A20 A21|A22 A23 A24 A25 A26 A27 A28 A29 */
   /*                         |   |   |   |   |   |   |   |   |   |   | */
   /*   11 row addresses---->A11 A12 A13 A14 A15 A16 A17 A18 A19 A20 A21 */
   /*   on corresponding */
   /*   column pins. */
   /* */
   /*   The row address is output on the address bus first, then the column  */
   /*   address.               */
   /* */
   /* - BSMA (Bank Select Multiplexed Address line) = 100 = A16-A18 */
   /* */
   /*    For Local bus accesses, this field is a "don't care". BNKSEL[0:2] */
   /*    are associated with this field and these pins are only used for  */
   /*    60x compatible mode for 60x bus transactions. */
   /* */
   /* - SDA10 ("A10 Control") = 001 = A11 is the address pin used for the */
   /*                                 activate command. */
   /* */
   /* - RFRC (ReFresh ReCovery) = 101 =  */
   /* */
   /*   Once a refresh request is granted, the memory controller begins  */
   /*   issuing auto-refresh commands to each device associated with the  */
   /*   refresh timer in one clock intervals. After the last REFRESH  */
   /*   command is issued, the memory controller waits for 7 clocks before  */
   /*   the ACTIVATE command and before normal read/write operations can  */
   /*   resume. */
   /* */
   /* - PRETOACT (PREcharge TO ACTivate interval) = 010 =  */
   /* */
   /*   Wait 2 clock cycles before an activate command. */
   /* */
   /* - ACTTORW (ACTivate TO Read/Write interval) = 010 = 2 clock cycles. */
   /*  */
   /* - BL (Burst Length) = 1 = SDRAM burst length is 8. This is programmed  */
   /*                           into the SDRAM via the MRS commmand. */
   /* */
   /* - LDOTOPRE (Last Data Out TO PREcharge) = 00 =  */
   /* */
   /*   The delay required from the last data out to precharge state is 0 clock */
   /*   cycles. */
   /*                                */
   /* - WRC (Write Recovery Time) = 01 =  */
   /* */
   /*   Time needed to elapse before another operation is 1 clock cycle. */
   /* */
   /* - EAMUX (External Address MUltipleXing) = 0 No external address */
   /*                                             multiplexing. */
   /* */
   /* - BUFCMD (Buffer Control Lines) = 0 = Normal timing for the control */
   /*                                       lines. */
   /* */
   /* - CL (CAS Latency) = 10 = CAS latency is 2. Two cycles after column */
   /*                           address is registered, data is valid. */
   /* */
   /*------------------------------------------------------------------------- */

   addis    r3,0,0x2886
   ori      r3,r3,0xA522
   stw      r3,LSDMR(r4)

   addis    r2,0,0x0400
/*   addi     r3,0,0xFFFF           */
   addi     r3,0,-1
   stb      r3,0(r2)         /* Write 0xFF to address 0x04000000 */


   /*------------------------------------------------------------------------- */
   /* Program the LSDMR keeping refresh services off and changing the */
   /* SDRAM operation to "CBR Refresh". This step is responsible for issuing */
   /* a minimum of 8 auto-refresh commands. This is done by the SDRAM machine */
   /* by issuing the CBR Refresh command by programming the OP field of the  */
   /* PSDMR register and writing 0xFF 8 times to an arbitrary address. */
   /*------------------------------------------------------------------------- */

   addis    r3,0,0x0886
   ori      r3,r3,0xA522
   stw      r3,LSDMR(r4)

   /*-------------------------------------------------- */
   /* Loop 8 times, writing 0xFF to address 0x04000000 */
   /*-------------------------------------------------- */

   addis r6,0,0
   ori   r6,r6,8
   mtspr CTR,r6           /* Load CTR with 8.  */

/*   addi  r3,0,0xFFFF */     /* Load 0xFs into r3 */
   addi  r3,0,-1      /* Load 0xFs into r3 */

write_loop1:

   stb   r3,0(r2)         /* Write 0 to address 0x04000000 */

   bc    16,0,write_loop1 /* Decrement CTR, then branch if the decremented CTR */
                          /* is not equal to 0       */

   /*------------------------------------------------------------------------- */
   /* Program the LSDMR again turning off refresh services and changing the */
   /* SDRAM operation to "Mode Register Write". Then do a single write to an */
   /* arbitrary location. The various fields that will be programmed in the  */
   /* mode register on the SDRAM were specified in fields of the LSDMR, like */
   /* the BR (burst length) and the CL (CAS Latency) field. */
   /*------------------------------------------------------------------------- */

   addis    r3,0,0x1886
   ori      r3,r3,0xA522
   stw      r3,LSDMR(r4)

/*   addi     r3,0,0xFFFF           */
   addi     r3,0,-1
   stb      r3,0(r2)         /* Write 0xFF to address 0x04000000 */

   /*------------------------------------------------------------------------- */
   /* Program the LSDMR one last time turning on refresh services and changing  */
   /* the SDRAM operation to "Normal Operation".  */
   /*------------------------------------------------------------------------- */

   addis    r3,0,0x4086
   ori      r3,r3,0xA522
   stw      r3,LSDMR(r4)

   /*------------------------------ */
   /* return from init_local_sdram */
   /*------------------------------ */

   bclr  20,0    /* jump unconditionally to effective address in Link */
                 /* register */




/*######################################################################## */
/* Function: init_60x_sdram */
/* */
/* Description: This function programs the 16 Mbyte SDRAM DIMM on the VADS  */
/*              board. This memory functions as the RAM for the 603 core and */
/*              is connected to the 60x bus. Base and Option Register 2 are  */
/*              used as well as the SDRAM Machine #1 and Chip Select #2. */
/* */
/*  History: */
/* */
/*  1/9/99    jay      Initial Release */
/* */
/*########################################################################         */

init_60x_sdram:


   /*################# */
   /* Program Refresh */
   /*################# */


   /*------------------------------------------------------------------------- */
   /* Program the 60x Bus Assigned SDRAM Refresh Timer (PSRT). */
   /*------------------------------------------------------------------------- */

   addi     r5,0,0x0010      /* load 0x10 or 16  */
   stb      r5,PSRT(r4)      /* store byte - bits[24-31] */


   /*######################## */
   /* Program Bank Registers */
   /*######################## */


   /*------------------------------------------------------------------------- */
   /* Base Register 2 (BR2): Bank 2 is assigned to the 16 Mbyte 60x SDRAM DIMM */
   /*                        that resides on the MPC8260 VADS board. The  */
   /*                        particulars are defined here.  */
   /* */
   /* - BA (Base Address) = 0x0000+0b for a total of 17 address bits. This  */
   /*                       value represents the upper 17 bits of the base  */
   /*                       address. */
   /* */
   /* - Bits 17-18 reserved. = 00 = cleared to 0. */
   /* */
   /* - PS (Port Size) = 00b = 64 bit port size */
   /* */
   /* - DECC (Data Error Correction and Checking) = 00 = Data errors checking */
   /*                                                    Disabled. */
   /* */
   /* - WP (Write Protect) = 0 = both read and write accesses are allowed */
   /* */
   /* - MS (Machine Select) = 010 = SDRAM Machine for 60x bus Selected */
   /* */
   /* - EMEMC (External Memory Controller Enable) = 0 =  */
   /* */
   /*    Accesses are handled by the memory controller according to MSEL. */
   /* */
   /* - ATOM (Atomic Operation) = 00 = The address space controlled by the  */
   /*                                  memory controller bank is not used for */
   /*                                  atomic operations. */
   /* */
   /* - DR (Delayed Read) = 0 = Normal operation. */
   /* */
   /* - V (Valid Bit) =  1 = Valid bit set */
   /*------------------------------------------------------------------------- */

   addis    r3,0,0x0000     /* R3 holds the value temporarily */
   ori      r3,r3,0x0041

   /*------------------------------------------------------------------------- */
   /* Option Register 2 (OR2) for SDRAM Machine use: further SDRAM definitions */
   /* */
   /* - USDAM (Upper SDRAM Address Mask) = 11111b =  */
   /* */
   /*    We've masked all 5 bits which defines a maximum 128 Mbyte block. */
   /*    Note that since the maximum SDRAM block size is 128 Mbyte, this field */
   /*    should always be 11111b. */
   /* */
   /* - SDAM (SDRAM Address Mask) = 1110000b = 16 Mbyte block. */
   /* */
   /* - LSDAM (Lower SDRAM Address Mask) = 00000b = Minimum of 1 Mbyte size. */
   /* */
   /* - BPD (Banks Per Device) = 00 = 2 internal banks per device. */
   /* */
   /* - ROWST (Row Start Address Bit) = 011 = A9 being the row start address  */
   /*                                         bit. */
   /* */
   /* - Bit 22 Reserved - set to 0. */
   /* */
   /* - NUMR (Number of Row Address Lines) = 010 = 11 row address lines. */
   /* */
   /* - PMSEL (Page Mode Select) = 1 = Reserved */
   /* */
   /* - IBID (Internal bank interleaving within same device disable) = 0 = */
   /*  */
   /*    Bank interleaving allowed. */
   /* */
   /* - AACKR (AACK/ release) = 0 = Normal operation. */
   /* */
   /* - Bits 29-31 Reserved - set to 0. */
   /*------------------------------------------------------------------------- */

   addis    r5,0,0xFF00     /* R5 holds the value temporarily */
   ori      r5,r5,0x0CA0

   /*------------------ */
   /* Write the values */
   /*------------------ */

   stw        r5,OR2(r4)
   stw        r3,BR2(r4)


   /*########################################## */
   /* Perform Initialization sequence to SDRAM */
   /*########################################## */

   /*------------------------------------------------------------------------- */
   /* Program the PowerPC SDRAM Mode Registr (PSDMR). This register is used */
   /* to configure operations pertaining to SDRAM. Program the PSDMR, turning  */
   /* off refresh services and changing the SDRAM operation to "Precharge all */
   /* banks". Then do a single write to an arbitrary location. Writing 0xFF  */
   /* to address 0 will do the trick. */
   /* */
   /* - Bit 0 is reserved. Set to 0. */
   /* */
   /* - RFEN (Refresh Enable) = 0 = Refresh services not required. This bit  */
   /*                               will be set later in this function as a  */
   /*                               last step. */
   /*  */
   /* - OP (SDRAM Operation) = 000 = Precharge all banks. */
   /* */
   /* - SDAM (Address Multiplex Size = 001 =  */
   /* */
   /*   Coming up the value for this field is one of the most confusing */
   /*   and non-intuitive steps in the SDRAM programming process. This is how */
   /*   it works... The SDRAM device is a 16 Mbit DIMM that has a data width  */
   /*   of 64 bits or 8 bytes. The bank size is 64 bits so the SDRAM will  */
   /*   ignore the least significant 3 bits. Given this information and  */
   /*   knowing that the number of row address lines is 11 and the column  */
   /*   addresses is 9 and also knowing that the row addresses must be  */
   /*   multiplexed, write out the following: */
   /* */
   /*   addresses ignored by SDRAM : A29 A30 A31 */
   /* */
   /*   column addresses: A20 A21 A22 A23 A24 A25 A26 A27 A28 */
   /* */
   /*   row addresses:   A9 A10 A11 A12 A13 A14 A15 A16 A17 A18 A19 */
   /* */
   /*   When the memory controller multiplexes the addresses it puts the  */
   /*   column addresses out on the actual physical pins that matches it's */
   /*   corresponding effective address bits. */
   /* */
   /*   According to the SDRAM Address Multiplexing table in the memory  */
   /*   controller section of the manual, SDAM selection of 001 fits the  */
   /*   address layout given above. Because it's the row that is multiplexed on */
   /*   designated column address pins, the row addresses are put out on the */
   /*   following column pins: */
   /* */
   /*   column ---------------------->  A20 A21 A22 A23 A24 A25 A26 A27 A28 */
   /*                                    |   |   |   |   |   |   |   |   | */
   /*   row address -----------> A9 A10 A11 A12 A13 A14 A15 A16 A17 A18 A19 */
   /*   on corresponding */
   /*   column pins. */
   /* */
   /*   Column pins A19, A18, and A17 were not connected to the DIMM device so */
   /*   A9 and A10 as well as the bank select address which is A8 are put out  */
   /*   on their respective pins because it was decided that these address  */
   /*   pins would be directly connected to the DIMM. That is, A10 of the real */
   /*   address is put out on the A10 pin, A9 on the A9 pin, etc. */
   /* */
   /*   As a final note on this subject, the row address is output on the  */
   /*   address bus first on the actual column address pins according to the  */
   /*   multiplexing scheme in the SDRAM address multiplexing table, then the  */
   /*   column address follows which makes up the most significant part of the */
   /*   real address which is connected to the memory device.  */
   /* */
   /*   Any entries in the multiplexing table that show a "-" means nothing is */
   /*   multiplexed on the physical pins shown on the top row of the table */
   /*   and therefore any part of the row address will also show up on it's  */
   /*   respective physical pin if this pin is not used for multiplexing. For  */
   /*   example, for a SDAM of 001, the multiplexing table shows that A13  */
   /*   through A0 are not multiplexed because they have "-" in their  */
   /*   respective boxes. This means that A13 through A5 of the real address */
   /*   will show up multiplexed on address pins A22 through A14 and A13  */
   /*   through A0 of the real address will also show up on their actual  */
   /*   respective physical pins. */
   /* */
   /* - BSMA (Bank Select Multiplexed Address line) = 011 = A15-A17 */
   /* */
   /*    BSMA determines how BNKSEL[0:2] will operate. This SDRAM has only 2  */
   /*    banks, so only BANKSEL2 is of concern because it's the least  */
   /*    significant bit. BANKSEL2 should be programmed to operate as physical  */
   /*    pin A17. Because we are multiplexing 9 column addresses, it will  */
   /*    actually behave as A8.  */
   /* */
   /* - SDA10 ("A10 Control") = 011 =  */
   /* */
   /*    A9 is the address pin used for the activate command. The A10/AP pin */
   /*    the SDRAM needs to act as a command line during on phase of the  */
   /*    access and as an address line in the other phase. The SDA10 pin on  */
   /*    MPC8260 takes care of this function. Instead of connecting the  */
   /*    A10/AP pin to A9 on the address bus, connect it to SDA10 and program */
   /*    SDA10 to function as A9. */
   /* */
   /* - RFRC (ReFresh ReCovery) = 101 =  */
   /* */
   /*   Once a  refresh request is granted, the memory controller begins  */
   /*   issuing auto-refresh commands to each device associated with the  */
   /*   refresh timer in one clock intervals. After the last REFRESH  */
   /*   command is issued, the memory controller waits for 7 clocks before  */
   /*   the ACTIVATE command and before normal read/write operations can  */
   /*   resume. */
   /* */
   /* - PRETOACT (PREcharge TO ACTivate interval) = 011 =  */
   /* */
   /*   Wait 3 clock cycles before an activate command. This parameter is */
   /*   determined by the requirements of the SDRAM at a particular clock  */
   /*   speed. */
   /* */
   /* - ACTTORW (ACTivate TO Read/Write interval) = 010 = 2 clock cycles. */
   /*  */
   /* - BL (Burst Length) = 0 = SDRAM burst length is 4. This is programmed  */
   /*                          into the SDRAM via the MRS commmand. */
   /* */
   /* - LDOTOPRE (Last Data Out TO PREcharge) = 01 =  */
   /* */
   /*   The delay required from the last data out to precharge state is 1 clock */
   /*   cycle. This parameter is determined by the requirements of the SDRAM at */
   /*   a particular clock speed. */
   /*                                */
   /* - WRC (Write Recovery Time) = 01 =  */
   /* */
   /*   Time needed to elapse before another operation is 1 clock cycle. This  */
   /*   parameter is determined by the requirements of the SDRAM at a  */
   /*   particular clock speed. */
   /* */
   /* - EAMUX (External Address MUltipleXing) = 0 No external address */
   /*                                             multiplexing. */
   /* */
   /* - BUFCMD (Buffer Control Lines) = 0 = Normal timing for the control */
   /*                                       lines. */
   /* */
   /* - CL (CAS Latency) = 10 =  */
   /* */
   /*    CAS latency is 2. Two cycles after column address is registered, data */
   /*    is valid. This parameter is determined by the requirements of the  */
   /*    SDRAM at a particular clock speed. */
   /* */
   /*------------------------------------------------------------------------- */

   addis    r3,0,0x296E
   ori      r3,r3,0xB452
   stw      r3,PSDMR(r4)

   addis    r0,0,0

   addi     r3,0,0x00FF      /* Load 0x000000FF into r3 */
   stb      r3,0(r0)         /* Write 0xFF to address 0 - bits [24-31] */


   /*------------------------------------------------------------------------- */
   /* Program the PSDMR keeping refresh services off and changing the */
   /* SDRAM operation to "CBR Refresh". This step is responsible for issuing */
   /* a minimum of 8 auto-refresh commands. This is done by the SDRAM machine */
   /* by issuing the CBR Refresh command by programming the OP field of the  */
   /* PSDMR register and writing 0xFF 8 times to an arbitrary address. */
   /*------------------------------------------------------------------------- */

   addis    r3,0,0x096E
   ori      r3,r3,0xB452
   stw      r3,PSDMR(r4)

   /*------------------------------------------ */
   /* Loop 8 times, writing 0xFF to address 0 */
   /*------------------------------------------ */

   addi  r6,0,0x0008
   mtspr CTR,r6             /* Load CTR with 8. The CTR special purpose */
                            /* is spr 9 */

   addi  r3,0,0x00FF      /* Load 0x000000FF into r3 */

write_loop:

   stb   r3,0(r0)         /* Write 0xFF to address 0 - bits [24-31] */

   bc    16,0,write_loop  /* Decrement CTR, then branch if the decremented CTR */
                          /* is not equal to 0       */

   /*------------------------------------------------------------------------- */
   /* Program the PSDMR again turning off refresh services and changing the */
   /* SDRAM operation to "Mode Register Write". Then do a single write to an */
   /* arbitrary location. The various fields that will be programmed in the  */
   /* mode register on the SDRAM were specified in fields of the PSDMR, like */
   /* the BR (burst length) and the CL (CAS Latency) field. */
   /*------------------------------------------------------------------------- */

   addis    r3,0,0x196E
   ori      r3,r3,0xB452
   stw      r3,PSDMR(r4)

   addi     r3,0,0x00FF      /* Load 0x000000FF into r3 */
   stb      r3,0(r0)         /* Write 0xFF to address 0 - bits [24-31] */

   /*------------------------------------------------------------------------- */
   /* Program the PSDMR one last time turning on refresh services and changing  */
   /* the SDRAM operation to "Normal Operation".  */
   /*------------------------------------------------------------------------- */

   addis    r3,0,0x416E
   ori      r3,r3,0xB452
   stw      r3,PSDMR(r4)

   /*---------------------------- */
   /* return from init_60x_sdram */
   /*---------------------------- */

   bclr  20,0    /* jump unconditionally to effective address in Link */
                 /* register */




	.Lstk:
	.long	__stack

